# -*- coding: utf-8 -*-
# @Author  : lx

"""
Datetime Library
~~~~~~~~~~~~~~~~~~~~~
"""

import datetime
import time
import re


class DateGo:
    @staticmethod
    def now_data():
        """
        获取当前时间
        :return:  %Y-%m-%d %H:%M:%S
        """
        return datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")


    @staticmethod
    def now_ymd()->list:
        """
        获取当前 年，月，日
        :return: list:[年,月,日]
        """
        now_date = datetime.datetime.now()
        y = str(now_date.year)
        m = str(now_date.month)
        d = str(now_date.day)
        return [y,m,d]


    @staticmethod                      # 把时间戳 转换为 年月日时分 格式
    def timec_change_dtime(time1):
        if len(str(time1))==13:
            time1 = int(time1) / 1000.0
        timeArray = time.localtime(int(time1))
        otherStyleTime = time.strftime("%Y-%m-%d %H:%M:%S", timeArray)
        return otherStyleTime

    
    @staticmethod                      # 负数时间戳转换/转换1970年之前的时间戳
    def netimec_to_dtime(timestamp,valid=True):
        """ 
        :param  timestamp: 负数时间戳
        :param  valid: 默认增加8小时
        :return:  年-月-日 时:分:秒
        if OverflowError , date value out of range: timestamp/1000
        """
        if valid:
            timestamp = int(timestamp) + 8*3600
        result = datetime.datetime(1970, 1, 1) + datetime.timedelta(seconds=int(timestamp))
        return result


    
    @staticmethod
    def dtime_to_timec(tss1):
        """
        把 年月日时分 转换为 时间戳格式
        :param tss1: str
        :return: timeStamp:int
        """
        timeArray = time.strptime(tss1, "%Y-%m-%d %H:%M:%S")
        timeStamp = int(time.mktime(timeArray))
        return timeStamp


    @staticmethod
    def yesterday_timec(day=1):
        """
        # 获取昨天开始的时间戳
        :param day: int
        """
        yesterday = datetime.date.today() - datetime.timedelta(days=day)
        yesterday_timec = int(time.mktime(time.strptime(str(yesterday), '%Y-%m-%d')))
        return yesterday_timec


    @staticmethod
    def difference_time(time1,time2):
        """
        两个时间的差 -> 秒
        """
        t1 = datetime.datetime.utcfromtimestamp(time1)
        t2 = datetime.datetime.utcfromtimestamp(time2)
        time_poor = (t1 - t2).seconds
        return time_poor


    @staticmethod
    def weibo_date(raw):
        """
         # 微博时间处理方法，转换为时间格式
        :param raw: str
        :return:   %Y-%m-%d %H:%M:%S
        """
        result = None
        if raw.find('分钟') != -1 or raw.find('小时') != -1 or raw.find('天') != -1:
            dn = datetime.datetime.now()
            dt = datetime.timedelta
            num = int(re.findall('\d{1,}', raw)[0])
            if '分钟' in raw:
                result = dn - dt(minutes=num)
            elif '小时' in raw:
                result = dn - dt(hours=num)
            elif '天' in raw:
                result = dn - dt(days=num)
            result = result.strftime("%Y-%m-%d %H:%M:%S")
            return result
        elif raw == '刚刚':
            return DateGo.now_data()
        elif len(raw)== 5 and raw[2] == '-':
            result = DateGo.now_ymd()[0] +'-' +raw +' 00:00:00'
            return result
        else:
            if len(raw)==10:
                raw = raw + ' 00:00:00'
                return raw
            else:
                return raw


    @staticmethod
    def youku_date(raw):
        """
        优酷时间处理方法
        """
        if raw == '昨天':
            return DateGo.date_befor_days(1)
        elif raw == '前天':
            return DateGo.date_befor_days(2)
        elif raw == '1周前' or raw=='一周前':
            return DateGo.date_befor_days(7)
        elif raw == '一月前':
            return DateGo.date_befor_days(30)
        elif '天前' in raw:
            return DateGo.youku_date_2(raw)
        elif '刚刚' == raw:
            return DateGo.now_data()
        elif '小时前' in raw:
            return DateGo.youku_date_2(raw)
        elif '分钟前' in raw:
            return DateGo.youku_date_2(raw)


    @staticmethod
    def youku_date_2(raw):
        if raw.find('分钟') != -1 or raw.find('小时') != -1 or raw.find('天') != -1:
            dn = datetime.datetime.now()
            dt = datetime.timedelta
            num = int(re.findall('\d{1,}', raw)[0])
            if  '分钟' in raw:
                result = dn - dt(minutes=num)
            elif '小时' in raw:
                result = dn - dt(hours=num)
            else:
                result = dn - dt(days=num)
            result = result.strftime("%Y-%m-%d %H:%M:%S")
            return result
        else:
            return raw


    @staticmethod
    def difference_timenow(date2):
        """
        当前时间和指定时间的差/-> 秒
        :param date2:
        :return:
        """
        date1 = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        date1 = datetime.datetime.strptime(date1, "%Y-%m-%d %H:%M:%S")
        date2 = datetime.datetime.strptime(date2, "%Y-%m-%d %H:%M:%S")
        difference = (date1 - date2).total_seconds()
        return difference


    @staticmethod
    def difference_time_two(date1,date2):
        """
        两个时间的差/-> 秒
        :return:
        """
        date1 = datetime.datetime.strptime(date1, "%Y-%m-%d %H:%M:%S")
        date2 = datetime.datetime.strptime(date2, "%Y-%m-%d %H:%M:%S")
        difference = (date1 - date2).total_seconds()
        return difference


    @staticmethod
    def date_befor_minutes(value):
        """ 分钟 前/后 的时间
        :param value:
        :return:
        """
        now_time = datetime.datetime.now()
        now_time = now_time - datetime.timedelta(minutes=int(value))
        return now_time.strftime('%Y-%m-%d %H:%M:%S')


    @staticmethod
    def date_befor_hours(value):
        """ 小时前/后 的时间
        :param value:
        :return:
        """
        now_time = datetime.datetime.now()
        now_time = now_time - datetime.timedelta(hours=int(value))
        return now_time.strftime('%Y-%m-%d %H:%M:%S')


    @staticmethod
    def date_befor_days(beforeOfDay:int):
        """ 几天前/后 的时间
        :param beforeOfDay:
        :return:
        """
        today = datetime.datetime.now()
        offset = datetime.timedelta(days=-beforeOfDay)
        re_date = (today + offset).strftime('%Y-%m-%d %H:%M:%S')
        return re_date


    @staticmethod
    def java_date(date,to_format="%Y-%m-%d %H:%M:%S",timezon ="CST"):
        """ java时间格式转换为python日期格式
        %a: 星期的缩写
        %b: 月份英文名的缩写
        %d: 日期(以01-31来表示)
        %Y: 年份(以四位数来表示)
        :param:　data:　Fri May 18 15:46:24 CST 2016
        :param:  timezon：时区 "CST"
        :return: to_format="%Y-%m-%d %H:%M:%S"
        """
        data_format = "%a %b %d %H:%M:%S {} %Y".format(timezon)
        time_struct = time.strptime(str(date),data_format)
        ctime = time.strftime(to_format,time_struct)
        return ctime




# -*- coding: utf-8 -*-

# TODO: b36\b62\b58\b91编码

import struct



# b36 看往上的说明，貌似应用于将数字转化成字符串进行保存？
def b36encode(num):
    alpha = '0123456789abcdefghijklmnopqrstuvwxyz'
    if not isinstance(num, int):
        raise TypeError('num must be an integer')
    if num < 0:
        return '-' + b36encode(-num)
    val = ''
    while num != 0:
        num, idx = divmod(num, len(alpha))
        val = alpha[idx] + val
    return val or '0'

def b36decode(val):
    return int(val, 36)

# s = b36encode(1111111111111111)
# print(s)
# s = b36decode(s)
# print(s)




def b62encode(num):
    alpha = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'
    if not isinstance(num, int):
        raise TypeError('num must be an integer')
    if num < 0:
        return '-' + b36encode(-num)
    val = ''
    while num != 0:
        num, idx = divmod(num, len(alpha))
        val = alpha[idx] + val
    return val or '0'

def b62decode(val):
    alpha = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'
    r = 0
    for idx,i in enumerate(val[::-1]):
        r += alpha.index(i) * (len(alpha) ** idx)
    return r

# s = b62encode(1111111111111111)
# print(s)
# s = b62decode(s)
# print(s)






base91_alphabet = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
                   'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
                   'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
                   'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
                   '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '!', '#', '$',
                   '%', '&', '(', ')', '*', '+', ',', '.', '/', ':', ';', '<', '=',
                   '>', '?', '@', '[', ']', '^', '_', '`', '{', '|', '}', '~', '"']

decode_table = dict((v, k) for k, v in enumerate(base91_alphabet))

def b91decode(encoded_str):
    ''' Decode Base91 string to a bytearray '''
    v = -1
    b = 0
    n = 0
    out = b''
    for strletter in encoded_str.decode():
        if not strletter in decode_table:
            continue
        c = decode_table[strletter]
        if (v < 0):
            v = c
        else:
            v += c * 91
            b |= v << n
            n += 13 if (v & 8191) > 88 else 14
            while True:
                out += struct.pack('B', b & 255)
                b >>= 8
                n -= 8
                if not n > 7:
                    break
            v = -1
    if v + 1:
        out += struct.pack('B', (b | v << n) & 255)
    return out

def b91encode(bindata):
    ''' Encode a bytearray to a Base91 string '''
    b = 0
    n = 0
    out = ''
    for count in range(len(bindata)):
        byte = bindata[count:count + 1]
        b |= struct.unpack('B', byte)[0] << n
        n += 8
        if n > 13:
            v = b & 8191
            if v > 88:
                b >>= 13
                n -= 13
            else:
                v = b & 16383
                b >>= 14
                n -= 14
            out += base91_alphabet[v % 91] + base91_alphabet[v // 91]
    if n:
        out += base91_alphabet[b % 91]
        if n > 7 or b > 90:
            out += base91_alphabet[b // 91]
    return out.encode()

# 统一输入输出的格式
# s = b91encode(b'asdfasdf')
# print(s)
# s = b91decode(s)
# print(s)



# b58
# 没有0，大写的O，大写的I，以及小写的l
alphabet = b'123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz'
alphalen = len(alphabet)

def b58encode_int(i, default_one=True):
    '''Encode an integer using Base58'''
    if not i and default_one:
        return alphabet[0:1]
    string = b""
    while i:
        i, idx = divmod(i, alphalen)
        string = alphabet[idx:idx+1] + string
    return string

def b58encode(v):
    nPad = len(v)
    v = v.lstrip(b'\0')
    nPad -= len(v)
    p, acc = 1, 0
    for c in reversed(v):
        acc += p * c
        p = p << 8
    result = b58encode_int(acc, default_one=False)
    return (alphabet[0:1] * nPad + result)

def b58decode_int(v):
    decimal = 0
    for char in v:
        decimal = decimal * alphalen + alphabet.index(char)
    return decimal

def b58decode(v):
    origlen = len(v)
    v = v.lstrip(alphabet[0:1])
    newlen = len(v)
    acc = b58decode_int(v)
    result = []
    while acc > 0:
        acc, mod = divmod(acc, 256)
        result.append(mod)
    return (b'\0' * (origlen - newlen) + bytes(reversed(result)))


# s = b58encode(b'asdfasdf')
# print(s)
# s = b58decode(s)
# print(s)


# -*- coding: utf-8 -*-
# @Author  : lx
# @IDE ：PyCharm

# 将js中的 eval 压缩加密的代码还原成压缩前的代码
def parse_evaljs(s):
    import re
    x = r"\}\(('(?:[^'\\]|\\.)*') *, *(\d+) *, *(\d+) *, *'((?:[^'\\]|\\.)*)'\.split\('\|'\) *, *(\d+) *, *(\{\})"
    p, a, c, k, e, d = re.findall(x, s)[0]
    p, a, c, e, k, d = eval(p), int(a), int(c), int(e), k.split('|'), {}
    def evaljs_code(p, a, c, k, e, d):
        def e(c):
            x = '' if c < a else e(int(c/a))
            c = c % a
            return x + (chr(c + 29) if c > 35 else '0123456789abcdefghijklmnopqrstuvwxyz'[c])
        for i in range(c): d[e(i)] = k[i] or e(i)
        return re.sub(r'\b(\w+)\b', lambda e: d.get(e.group(0)) or e.group(0), p)
    return evaljs_code(p, a, c, k, e, d)



# -*- coding: utf-8 -*-
# @Author  : lx

import re

def html_format(string):
    """ html去除标签 """
    dr = re.compile(r'<[^>]+>', re.S)
    not_format = dr.sub('', string)
    return not_format


def jsonp_to_json(jsonp):
    result = re.findall(r'\w+[(]{1}(.*)[)]{1}',jsonp,re.S)
    return result


def re_xpath(node,compile):
    """
    :param compile: './/span[re:match(@class, "allstar(\d0)")]/@class'
    """
    namespaces = {"re": "http://exslt.org/regular-expressions"}
    result = node.xpath(compile, namespaces=namespaces)
    return result


def url_parse(url:str):
    # 提取url中的params参数，返回item
    p = url.split('?')[1]
    item = {}
    if '&' in p:
        param = p.split('&')
        for v in param:
            k = v.split('=')
            item[k[0]] = k[1]
    else:
         k = p.split('=')
         item[k[0]] = k[1]
    return item





# -*- coding: utf-8 -*-
# @Time    : 2021/12/14 12:47
# @Author  : lx
# @IDE ：PyCharm
import random

def get_tracks(distance):
    tracks = []
    y = 0
    v = 0
    t = 1
    current = 0
    mid = distance * 3 / 4
    exceed = random.randint(40, 50)
    z = random.randint(30, 150)

    while current < (distance + exceed):
        if current < mid / 2:
            a = 2
        elif current < mid:
            a = 3
        else:
            a = -3
        a /= 2
        v0 = v
        s = v0 * t + 0.5 * a * (t * t)
        current += int(s)
        v = v0 + a * t

        y += random.randint(-3, 3)
        z = z + random.randint(6, 8)
        tracks.append([min(current, (distance + exceed)), y, z])

    while exceed > 0:
        exceed -= random.randint(0, 5)
        y += random.randint(-1, 3)
        z = z + random.randint(6, 8)
        tracks.append([min(current, (distance + exceed)), y, z])
    # return tracks
    track_str = ''
    for x in tracks:
        x = [str(i) for i in x]
        x = '|'.join(x)
        track_str += (x + ',')
    return track_str[:-1]


# test:
# dis_x =500
# print(get_tracks(int(dis_x * (392 / 118))))





#-*- coding:utf-8 -*-
# @Author  : lx

# TODO 一些加密算法依赖于其他第三方库，所以进行了注释
# TODO 如有需要，自行修改和安装。

__all__ = ['DateGo','create_root_node','get_ua','re_xpath','url_parse',
           'copy_headers_dict','html_format','jsonp_to_json',
           'get_tracks','parse_evaljs',
           'encoding','cipher',
           'encrypt','get_hamc','get_md5','rc4',
           #'aes_encrypt','aes_decrypt','des_encrypt','des_decrypt',
           ]

from .lxdate import DateGo
from .lxml_check import create_root_node
from .lxheader import *
from .lxtools import *
from .js import *
from .tracks import *
from .encrypt.md5 import get_md5
from .encrypt.hmac import get_hamc
#from .encrypt.aes import aes_encrypt,aes_decrypt
#from .encrypt.des import des_encrypt,des_decrypt
from .encrypt.rc4 import rc4
from .cipher import *
from .encoding import *
